#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/7/21 14:22
Desc: 郑州商品交易所-交易数据-历史行情下载-期权历史行情下载
http://www.czce.com.cn/cn/jysj/lshqxz/H770319index_1.htm
自 20200101 起，成交量、空盘量、成交额、行权量均为单边计算
郑州商品交易所-期权上市时间表
"SR": "20170419"
"CF": "20190410"
"TA": "20191216"
"MA": "20191217"
"RM": "20200116"
"ZC": "20200630"
"""
from io import StringIO
import warnings

import pandas as pd
import requests


def option_czce_hist(symbol: str = "SR", year: str = "2021") -> pd.DataFrame:
    """
    郑州商品交易所-交易数据-历史行情下载-期权历史行情下载
    http://www.czce.com.cn/cn/jysj/lshqxz/H770319index_1.htm
    :param symbol: choice of {"白糖": "SR", "棉花": "CF", "PTA": "TA", "甲醇": "MA", "菜籽粕": "RM", "动力煤": "ZC"}
    :type symbol: str
    :param year: 需要获取数据的年份, 注意品种的上市时间
    :type year: str
    :return: 指定年份的日频期权数据
    :rtype: pandas.DataFrame
    """
    symbol_year_dict = {
        "SR": "2017",
        "CF": "2019",
        "TA": "2019",
        "MA": "2019",
        "RM": "2020",
        "ZC": "2020",
    }
    if int(symbol_year_dict[symbol]) > int(year):
        warnings.warn(f"{year} year, symbol {symbol} is not on trade")
        return None
    warnings.warn("正在下载中, 请稍等")
    url = f'http://www.czce.com.cn/cn/DFSStaticFiles/Option/2021/OptionDataAllHistory/{symbol}OPTIONS{year}.txt'
    r = requests.get(url)
    option_df = pd.read_table(StringIO(r.text), skiprows=1, sep="|", low_memory=False)
    return option_df


if __name__ == "__main__":
    option_czce_hist_df = option_czce_hist(symbol="ZC", year="2021")
    print(option_czce_hist_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/10/27 20:30
Desc: 天天基金网-基金数据-规模份额
https://fund.eastmoney.com/data/cyrjglist.html
"""
import pandas as pd
import requests

from byteapp_ak.utils import demjson


def fund_scale_change_em() -> pd.DataFrame:
    """
    天天基金网-基金数据-规模份额-规模变动
    https://fund.eastmoney.com/data/gmbdlist.html
    :return: 规模变动
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/data/FundDataPortfolio_Interface.aspx"
    params = {
        "dt": "9",
        "pi": "1",
        "pn": "50",
        "mc": "hypzDetail",
        "st": "desc",
        "sc": "reportdate",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("{"): -1])
    total_page = data_json["pages"]
    big_df = pd.DataFrame()
    for page in range(1, int(total_page) + 1):
        params.update({"pi": page})
        r = requests.get(url, params=params)
        data_text = r.text
        data_json = demjson.decode(data_text[data_text.find("{"): -1])
        temp_df = pd.DataFrame(data_json["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df["index"] + 1
    big_df.columns = [
        "序号",
        "截止日期",
        "基金家数",
        "期间申购",
        "期间赎回",
        "期末总份额",
        "期末净资产",
    ]
    big_df["截止日期"] = pd.to_datetime(big_df["截止日期"], errors="coerce").dt.date
    big_df["基金家数"] = pd.to_numeric(big_df["基金家数"], errors="coerce")
    big_df["期间申购"] = pd.to_numeric(big_df["期间申购"].str.replace(",", ""), errors="coerce")
    big_df["期间赎回"] = pd.to_numeric(big_df["期间赎回"].str.replace(",", ""), errors="coerce")
    big_df["期末总份额"] = pd.to_numeric(big_df["期末总份额"].str.replace(",", ""), errors="coerce")
    big_df["期末净资产"] = pd.to_numeric(big_df["期末净资产"].str.replace(",", ""), errors="coerce")
    return big_df


def fund_hold_structure_em() -> pd.DataFrame:
    """
    天天基金网-基金数据-规模份额-持有人结构
    https://fund.eastmoney.com/data/cyrjglist.html
    :return: 持有人结构
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/data/FundDataPortfolio_Interface.aspx"
    params = {
        "dt": "11",
        "pi": "1",
        "pn": "50",
        "mc": "hypzDetail",
        "st": "desc",
        "sc": "reportdate",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("{"): -1])
    total_page = data_json["pages"]
    big_df = pd.DataFrame()
    for page in range(1, int(total_page) + 1):
        params.update({"pi": page})
        r = requests.get(url, params=params)
        data_text = r.text
        data_json = demjson.decode(data_text[data_text.find("{"): -1])
        temp_df = pd.DataFrame(data_json["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df["index"] + 1
    big_df.columns = [
        "序号",
        "截止日期",
        "基金家数",
        '机构持有比列',
        '个人持有比列',
        '内部持有比列',
        '总份额',
    ]
    big_df["截止日期"] = pd.to_datetime(big_df["截止日期"], errors="coerce").dt.date
    big_df["基金家数"] = pd.to_numeric(big_df["基金家数"], errors="coerce")
    big_df["机构持有比列"] = pd.to_numeric(big_df["机构持有比列"], errors="coerce")
    big_df["个人持有比列"] = pd.to_numeric(big_df["个人持有比列"], errors="coerce")
    big_df["内部持有比列"] = pd.to_numeric(big_df["内部持有比列"], errors="coerce")
    big_df["总份额"] = pd.to_numeric(big_df["总份额"].str.replace(",", ""), errors="coerce")
    return big_df


if __name__ == "__main__":
    fund_scale_change_em_df = fund_scale_change_em()
    print(fund_scale_change_em_df)

    fund_hold_structure_em_df = fund_hold_structure_em()
    print(fund_hold_structure_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/10/14 22:00
Desc: 巨潮资讯-个股-公司概况
https://webapi.cninfo.com.cn/#/company
"""

import pandas as pd
import py_mini_racer
import requests

from byteapp_ak.datasets import get_ths_js


def _get_file_content_ths(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_profile_cninfo(symbol: str = "600030") -> pd.DataFrame:
    """
    巨潮资讯-个股-公司概况
    https://webapi.cninfo.com.cn/#/company
    :param symbol: 股票代码
    :type symbol: str
    :return: 公司概况
    :rtype: pandas.DataFrame
    :raise: Exception，如果服务器返回的数据无法被解析
    """
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1133"
    params = {
        "scode": symbol,
    }
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Accept-Enckey": mcode,
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, params=params, headers=headers)
    data_json = r.json()
    columns = [
        "公司名称",
        "英文名称",
        "曾用简称",
        "A股代码",
        "A股简称",
        "B股代码",
        "B股简称",
        "H股代码",
        "H股简称",
        "入选指数",
        "所属市场",
        "所属行业",
        "法人代表",
        "注册资金",
        "成立日期",
        "上市日期",
        "官方网站",
        "电子邮箱",
        "联系电话",
        "传真",
        "注册地址",
        "办公地址",
        "邮政编码",
        "主营业务",
        "经营范围",
        "机构简介",
    ]
    count = data_json["count"]
    if count == 1:
        # 有公司概况的
        redundant_json = data_json["records"][0]
        records_json = {}
        i = 0
        for k, v in redundant_json.items():
            if i == (len(redundant_json) - 4):
                break
            records_json[k] = v
            i += 1
        del i
        temp_df = pd.Series(records_json).to_frame().T
        temp_df.columns = columns
    elif count == 0:
        # 没公司概况的
        temp_df = pd.DataFrame(columns=columns)
    else:
        raise Exception("数据错误！")
    return temp_df


if __name__ == "__main__":
    stock_profile_cninfo_df = stock_profile_cninfo(symbol="600030")
    print(stock_profile_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/12/22 20:00
Desc: 胡润排行榜
https://www.hurun.net/
"""
import warnings

import pandas as pd
import requests
from bs4 import BeautifulSoup


def hurun_rank(indicator: str = "胡润百富榜", year: str = "2023") -> pd.DataFrame:
    """
    胡润排行榜
    https://www.hurun.net/CN/HuList/Index?num=3YwKs889SRIm
    :param indicator: choice of {"胡润百富榜", "胡润全球富豪榜", "胡润印度榜", "胡润全球独角兽榜", "全球瞪羚企业榜", "胡润Under30s创业领袖榜", "胡润中国500强民营企业", "胡润世界500强", "胡润艺术榜"}
    :type indicator: str
    :param year: 指定年份; {"胡润百富榜": "2014-至今", "胡润全球富豪榜": "2019-至今", "胡润印度榜": "2018-至今", "胡润全球独角兽榜": "2019-至今", "中国瞪羚企业榜": "2021-至今", "全球瞪羚企业榜": "2021-至今", "胡润Under30s创业领袖榜": "2019-至今", "胡润中国500强民营企业": "2019-至今", "胡润世界500强": "2020-至今", "胡润艺术榜": "2019-至今"}
    :type year: str
    :return: 指定 indicator 和 year 的数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.hurun.net/zh-CN/Rank/HsRankDetails?pagetype=rich"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    url_list = []
    for item in soup.find_all("ul", attrs={"class": "dropdown-menu"}):
        for inner_item in item.find_all("a"):
            url_list.append("https://www.hurun.net" + inner_item["href"])
    name_list = []
    for item in soup.find_all("ul", attrs={"class": "dropdown-menu"}):
        for inner_item in item.find_all("a"):
            name_list.append(inner_item.text.strip())

    name_url_map = dict(zip(name_list, url_list))
    r = requests.get(name_url_map[indicator])
    soup = BeautifulSoup(r.text, "lxml")
    code_list = [
        item["value"].split("=")[2]
        for item in soup.find(attrs={"id": "exampleFormControlSelect1"}).find_all(
            "option"
        )
    ]
    year_list = [
        item.text.split(" ")[0]
        for item in soup.find(attrs={"id": "exampleFormControlSelect1"}).find_all(
            "option"
        )
    ]
    year_code_map = dict(zip(year_list, code_list))
    params = {
        "num": year_code_map[year],
        "search": "",
        "offset": "0",
        "limit": "20000",
    }
    if year == "2018":
        warnings.warn("正在下载中")
        offset = 0
        limit = 20
        big_df = pd.DataFrame()
        while offset < 2200:
            try:
                params.update(
                    {
                        "offset": offset,
                        "limit": limit,
                    }
                )
                url = "https://www.hurun.net/zh-CN/Rank/HsRankDetailsList"
                r = requests.get(url, params=params)
                data_json = r.json()
                temp_df = pd.DataFrame(data_json["rows"])
                offset = offset + 20
                big_df = pd.concat([big_df, temp_df], ignore_index=True)
            except requests.exceptions.JSONDecodeError as e:
                offset = offset + 40
                continue
        big_df.rename(
            columns={
                "hs_Rank_Rich_Ranking": "排名",
                "hs_Rank_Rich_Wealth": "财富",
                "hs_Rank_Rich_Ranking_Change": "排名变化",
                "hs_Rank_Rich_ChaName_Cn": "姓名",
                "hs_Rank_Rich_ComName_Cn": "企业",
                "hs_Rank_Rich_Industry_Cn": "行业",
            },
            inplace=True,
        )
        big_df = big_df[
            [
                "排名",
                "财富",
                "姓名",
                "企业",
                "行业",
            ]
        ]
        return big_df
    url = "https://www.hurun.net/zh-CN/Rank/HsRankDetailsList"
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["rows"])
    if indicator == "胡润百富榜":
        temp_df.rename(
            columns={
                "hs_Rank_Rich_Ranking": "排名",
                "hs_Rank_Rich_Wealth": "财富",
                "hs_Rank_Rich_Ranking_Change": "排名变化",
                "hs_Rank_Rich_ChaName_Cn": "姓名",
                "hs_Rank_Rich_ComName_Cn": "企业",
                "hs_Rank_Rich_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "排名",
                "财富",
                "姓名",
                "企业",
                "行业",
            ]
        ]
    elif indicator == "胡润全球富豪榜":
        temp_df.rename(
            columns={
                "hs_Rank_Global_Ranking": "排名",
                "hs_Rank_Global_Wealth": "财富",
                "hs_Rank_Global_Ranking_Change": "排名变化",
                "hs_Rank_Global_ChaName_Cn": "姓名",
                "hs_Rank_Global_ComName_Cn": "企业",
                "hs_Rank_Global_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "排名",
                "财富",
                "姓名",
                "企业",
                "行业",
            ]
        ]
    elif indicator == "胡润印度榜":
        temp_df.rename(
            columns={
                "hs_Rank_India_Ranking": "排名",
                "hs_Rank_India_Wealth": "财富",
                "hs_Rank_India_Ranking_Change": "排名变化",
                "hs_Rank_India_ChaName_Cn": "姓名",
                "hs_Rank_India_ComName_Cn": "企业",
                "hs_Rank_India_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "排名",
                "财富",
                "姓名",
                "企业",
                "行业",
            ]
        ]
    elif indicator == "胡润全球独角兽榜":
        temp_df.rename(
            columns={
                "hs_Rank_Unicorn_Ranking": "排名",
                "hs_Rank_Unicorn_Wealth": "财富",
                "hs_Rank_Unicorn_Ranking_Change": "排名变化",
                "hs_Rank_Unicorn_ChaName_Cn": "姓名",
                "hs_Rank_Unicorn_ComName_Cn": "企业",
                "hs_Rank_Unicorn_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "排名",
                "财富",
                "姓名",
                "企业",
                "行业",
            ]
        ]
    elif indicator == "中国瞪羚企业榜":
        temp_df.rename(
            columns={
                "hs_Rank_CGazelles_ComHeadquarters_Cn": "企业总部",
                "hs_Rank_CGazelles_Name_Cn": "掌门人/联合创始人",
                "hs_Rank_CGazelles_ComName_Cn": "企业信息",
                "hs_Rank_CGazelles_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "企业信息",
                "掌门人/联合创始人",
                "企业总部",
                "行业",
            ]
        ]
    elif indicator == "全球瞪羚企业榜":
        temp_df.rename(
            columns={
                "hs_Rank_GGazelles_ComHeadquarters_Cn": "企业总部",
                "hs_Rank_GGazelles_Name_Cn": "掌门人/联合创始人",
                "hs_Rank_GGazelles_ComName_Cn": "企业信息",
                "hs_Rank_GGazelles_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "企业信息",
                "掌门人/联合创始人",
                "企业总部",
                "行业",
            ]
        ]
    elif indicator == "胡润Under30s创业领袖榜":
        temp_df.rename(
            columns={
                "hs_Rank_U30_ComHeadquarters_Cn": "企业总部",
                "hs_Rank_U30_ChaName_Cn": "姓名",
                "hs_Rank_U30_ComName_Cn": "企业信息",
                "hs_Rank_U30_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "姓名",
                "企业信息",
                "企业总部",
                "行业",
            ]
        ]
    elif indicator == "胡润中国500强民营企业":
        temp_df.rename(
            columns={
                "hs_Rank_CTop500_Ranking": "排名",
                "hs_Rank_CTop500_Wealth": "企业估值",
                "hs_Rank_CTop500_Ranking_Change": "排名变化",
                "hs_Rank_CTop500_ChaName_Cn": "CEO",
                "hs_Rank_CTop500_ComName_Cn": "企业信息",
                "hs_Rank_CTop500_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "排名",
                "排名变化",
                "企业估值",
                "企业信息",
                "CEO",
                "行业",
            ]
        ]
    elif indicator == "胡润世界500强":
        temp_df.rename(
            columns={
                "hs_Rank_GTop500_Ranking": "排名",
                "hs_Rank_GTop500_Wealth": "企业估值",
                "hs_Rank_GTop500_Ranking_Change": "排名变化",
                "hs_Rank_GTop500_ChaName_Cn": "CEO",
                "hs_Rank_GTop500_ComName_Cn": "企业信息",
                "hs_Rank_GTop500_Industry_Cn": "行业",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "排名",
                "排名变化",
                "企业估值",
                "企业信息",
                "CEO",
                "行业",
            ]
        ]
    elif indicator == "胡润艺术榜":
        temp_df.rename(
            columns={
                "hs_Rank_Art_Ranking": "排名",
                "hs_Rank_Art_Turnover": "成交额",
                "hs_Rank_Art_Ranking_Change": "排名变化",
                "hs_Rank_Art_Name_Cn": "姓名",
                "hs_Rank_Art_Age": "年龄",
                "hs_Rank_Art_ArtCategory_Cn": "艺术类别",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "排名",
                "排名变化",
                "成交额",
                "姓名",
                "年龄",
                "艺术类别",
            ]
        ]
    return temp_df


if __name__ == "__main__":
    hurun_rank_df = hurun_rank(indicator="胡润百富榜", year="2023")
    print(hurun_rank_df)

    hurun_rank_df = hurun_rank(indicator="胡润全球富豪榜", year="2023")
    print(hurun_rank_df)

    hurun_rank_df = hurun_rank(indicator="胡润全球独角兽榜", year="2023")
    print(hurun_rank_df)

    hurun_rank_df = hurun_rank(indicator="胡润印度榜", year="2021")
    print(hurun_rank_df)

    hurun_rank_df = hurun_rank(indicator="全球瞪羚企业榜", year="2021")
    print(hurun_rank_df)

    hurun_rank_df = hurun_rank(indicator="胡润Under30s创业领袖榜", year="2021")
    print(hurun_rank_df)

    hurun_rank_df = hurun_rank(indicator="胡润世界500强", year="2022")
    print(hurun_rank_df)

    hurun_rank_df = hurun_rank(indicator="胡润艺术榜", year="2023")
    print(hurun_rank_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/20 23:00
Desc: 东方财富-数据中心-中国油价
https://data.eastmoney.com/cjsj/oil_default.html
"""
import pandas as pd
import requests


def energy_oil_hist() -> pd.DataFrame:
    """
    汽柴油历史调价信息
    https://data.eastmoney.com/cjsj/oil_default.html
    :return: 汽柴油历史调价信息
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPTA_WEB_YJ_BD",
        "columns": "ALL",
        "sortColumns": "dim_date",
        "sortTypes": "-1",
        "token": "894050c76af8597a853f5b408b759f5d",
        "pageNumber": "1",
        "pageSize": "1000",
        "source": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1652959763351",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = ["调整日期", "汽油价格", "柴油价格", "汽油涨跌", "柴油涨跌"]
    temp_df["调整日期"] = pd.to_datetime(temp_df["调整日期"], errors="coerce").dt.date
    temp_df["汽油价格"] = pd.to_numeric(temp_df["汽油价格"], errors="coerce")
    temp_df["柴油价格"] = pd.to_numeric(temp_df["柴油价格"], errors="coerce")
    temp_df["汽油涨跌"] = pd.to_numeric(temp_df["汽油涨跌"], errors="coerce")
    temp_df["柴油涨跌"] = pd.to_numeric(temp_df["柴油涨跌"], errors="coerce")
    temp_df.sort_values(by=["调整日期"], inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def energy_oil_detail(date: str = "20220517") -> pd.DataFrame:
    """
    全国各地区的汽油和柴油油价
    https://data.eastmoney.com/cjsj/oil_default.html
    :param date: 可以调用 ak.energy_oil_hist() 得到可以获取油价的调整时间
    :type date: str
    :return: oil price at specific date
    :rtype: pandas.DataFrame
    """
    date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPTA_WEB_YJ_JH",
        "columns": "ALL",
        "filter": f"(dim_date='{date}')",
        "sortColumns": "cityname",
        "sortTypes": "1",
        "token": "894050c76af8597a853f5b408b759f5d",
        "pageNumber": "1",
        "pageSize": "1000",
        "source": "WEB",
        "_": "1652959763351",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"]).iloc[:, 1:]
    temp_df.columns = [
        "日期",
        "地区",
        "V_0",
        "V_92",
        "V_95",
        "V_89",
        "ZDE_0",
        "ZDE_92",
        "ZDE_95",
        "ZDE_89",
        "QE_0",
        "QE_92",
        "QE_95",
        "QE_89",
        "首字母",
    ]
    del temp_df["首字母"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["V_0"] = pd.to_numeric(temp_df["V_0"], errors="coerce")
    temp_df["V_92"] = pd.to_numeric(temp_df["V_92"], errors="coerce")
    temp_df["V_95"] = pd.to_numeric(temp_df["V_95"], errors="coerce")
    temp_df["V_89"] = pd.to_numeric(temp_df["V_89"], errors="coerce")
    temp_df["ZDE_0"] = pd.to_numeric(temp_df["ZDE_0"], errors="coerce")
    temp_df["ZDE_92"] = pd.to_numeric(temp_df["ZDE_92"], errors="coerce")
    temp_df["ZDE_95"] = pd.to_numeric(temp_df["ZDE_95"], errors="coerce")
    temp_df["ZDE_89"] = pd.to_numeric(temp_df["ZDE_89"], errors="coerce")
    temp_df["QE_0"] = pd.to_numeric(temp_df["QE_0"], errors="coerce")
    temp_df["QE_92"] = pd.to_numeric(temp_df["QE_92"], errors="coerce")
    temp_df["QE_95"] = pd.to_numeric(temp_df["QE_95"], errors="coerce")
    temp_df["QE_89"] = pd.to_numeric(temp_df["QE_89"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    energy_oil_hist_df = energy_oil_hist()
    print(energy_oil_hist_df)

    energy_oil_detail_df = energy_oil_detail(date="20240118")
    print(energy_oil_detail_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/22 17:30
Desc: 申万宏源研究-行业分类
http://www.swhyresearch.com/institute_sw/allIndex/downloadCenter/industryType
"""

import io

import pandas as pd
import requests

from byteapp_ak.utils.cons import headers


def stock_industry_clf_hist_sw() -> pd.DataFrame:
    """
    申万宏源研究-行业分类-全部行业分类
    https://www.swsresearch.com/swindex/pdf/SwClass2021/StockClassifyUse_stock.xls
    :return: 个股行业分类变动历史
    :rtype: pandas.DataFrame
    """
    url = "https://www.swsresearch.com/swindex/pdf/SwClass2021/StockClassifyUse_stock.xls"  # 此处为 https
    r = requests.get(url, headers=headers)
    temp_df = pd.read_excel(
        io.BytesIO(r.content), dtype={"股票代码": "str", "行业代码": "str"}
    )
    temp_df.rename(
        columns={
            "股票代码": "symbol",
            "计入日期": "start_date",
            "行业代码": "industry_code",
            "更新日期": "update_time",
        },
        inplace=True,
    )
    temp_df["start_date"] = pd.to_datetime(
        temp_df["start_date"], errors="coerce"
    ).dt.date
    temp_df["update_time"] = pd.to_datetime(
        temp_df["update_time"], errors="coerce"
    ).dt.date
    return temp_df


if __name__ == "__main__":
    stock_industry_clf_hist_sw_df = stock_industry_clf_hist_sw()
    print(stock_industry_clf_hist_sw_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/4/29 17:01
Desc: 雪球-行情中心-沪深股市-内部交易
https://xueqiu.com/hq/insider
"""
import pandas as pd
import requests


def stock_inner_trade_xq() -> pd.DataFrame:
    """
    雪球-行情中心-沪深股市-内部交易
    https://xueqiu.com/hq/insider
    :return: 内部交易
    :rtype: pandas.DataFrame
    """
    url = "https://xueqiu.com/service/v5/stock/f10/cn/skholderchg"
    params = {
        'size': '100000',
        'page': '1',
        'extend': 'true',
        '_': '1651223013040',
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "xueqiu.com",
        "Pragma": "no-cache",
        "Referer": "https://xueqiu.com/hq",
        "sec-ch-ua": '" Not A;Brand";v="99", "Chromium";v="100", "Google Chrome";v="100"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["items"])
    temp_df.columns = [
        '股票代码',
        '股票名称',
        '变动人',
        '-',
        '变动日期',
        '变动股数',
        '成交均价',
        '变动后持股数',
        '与董监高关系',
        '董监高职务',
    ]
    temp_df = temp_df[[
        '股票代码',
        '股票名称',
        '变动日期',
        '变动人',
        '变动股数',
        '成交均价',
        '变动后持股数',
        '与董监高关系',
        '董监高职务',
    ]]
    temp_df['变动日期'] = pd.to_datetime(temp_df['变动日期'], unit="ms").dt.date
    temp_df['变动股数'] = pd.to_numeric(temp_df['变动股数'], errors="coerce")
    temp_df['成交均价'] = pd.to_numeric(temp_df['成交均价'], errors="coerce")
    temp_df['变动后持股数'] = pd.to_numeric(temp_df['变动后持股数'], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_inner_trade_xq_df = stock_inner_trade_xq()
    print(stock_inner_trade_xq_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/31 16:30
Desc: 东方财富网-行情中心-期权市场
https://quote.eastmoney.com/center/qqsc.html
"""

import pandas as pd
import requests


def option_current_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-期权市场
    https://quote.eastmoney.com/center/qqsc.html
    :return: 期权价格
    :rtype: pandas.DataFrame
    """
    url = "https://23.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "200000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:10,m:12,m:140,m:141,m:151,m:163,m:226",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,"
        "f23,f24,f25,f22,f28,f11,f62,f128,f136,f115,f152,f133,f108,f163,f161,f162",
        "_": "1606225274063",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"] + 1
    temp_df.columns = [
        "序号",
        "_",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "_",
        "_",
        "_",
        "_",
        "_",
        "代码",
        "市场标识",
        "名称",
        "_",
        "_",
        "今开",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "昨结",
        "_",
        "持仓量",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "行权价",
        "剩余日",
        "日增",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "成交量",
            "成交额",
            "持仓量",
            "行权价",
            "剩余日",
            "日增",
            "昨结",
            "今开",
            "市场标识",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["持仓量"] = pd.to_numeric(temp_df["持仓量"], errors="coerce")
    temp_df["行权价"] = pd.to_numeric(temp_df["行权价"], errors="coerce")
    temp_df["剩余日"] = pd.to_numeric(temp_df["剩余日"], errors="coerce")
    temp_df["日增"] = pd.to_numeric(temp_df["日增"], errors="coerce")
    temp_df["昨结"] = pd.to_numeric(temp_df["昨结"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    option_current_cffex_em_df = option_current_cffex_em()
    big_df = pd.concat(objs=[temp_df, option_current_cffex_em_df], ignore_index=True)
    big_df["序号"] = range(1, len(big_df) + 1)
    return big_df


def option_current_cffex_em() -> pd.DataFrame:
    url = "https://futsseapi.eastmoney.com/list/option/221"
    params = {
        "orderBy": "zdf",
        "sort": "desc",
        "pageSize": "20000",
        "pageIndex": "0",
        "token": "58b2fa8f54638b60b87d69b31969089c",
        "field": "dm,sc,name,p,zsjd,zde,zdf,f152,vol,cje,ccl,xqj,syr,rz,zjsj,o",
        "blockName": "callback",
        "_:": "1706689899924",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["list"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"] + 1
    temp_df.rename(
        columns={
            "index": "序号",
            "rz": "日增",
            "dm": "代码",
            "zsjd": "-",
            "ccl": "持仓量",
            "syr": "剩余日",
            "o": "今开",
            "p": "最新价",
            "sc": "市场标识",
            "xqj": "行权价",
            "vol": "成交量",
            "name": "名称",
            "zde": "涨跌额",
            "zdf": "涨跌幅",
            "zjsj": "昨结",
            "cje": "成交额",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "成交量",
            "成交额",
            "持仓量",
            "行权价",
            "剩余日",
            "日增",
            "昨结",
            "今开",
            "市场标识",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["持仓量"] = pd.to_numeric(temp_df["持仓量"], errors="coerce")
    temp_df["行权价"] = pd.to_numeric(temp_df["行权价"], errors="coerce")
    temp_df["剩余日"] = pd.to_numeric(temp_df["剩余日"], errors="coerce")
    temp_df["日增"] = pd.to_numeric(temp_df["日增"], errors="coerce")
    temp_df["昨结"] = pd.to_numeric(temp_df["昨结"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    option_current_em_df = option_current_em()
    print(option_current_em_df)

    option_current_cffex_em_df = option_current_cffex_em()
    print(option_current_cffex_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/11/10 22:52
Desc: 数据接口初始化
"""
from byteapp_ak.pro import client
from byteapp_ak.utils import token_process


def pro_api(token=''):
    """
    初始化 pro API,第一次可以通过ak.set_token('your token')来记录自己的token凭证，临时token可以通过本参数传入
    """
    if token == '' or token is None:
        token = token_process.get_token()
    if token is not None and token != '':
        pro = client.DataApi(token)
        return pro
    else:
        raise Exception('api init error.')


if __name__ == '__main__':
    pro_test = pro_api()
    variety_all_df = pro_test.variety_all()
    print(variety_all_df)
    variety_no_futures_df = pro_test.variety_no_futures(symbol="RB", date="2018-08-08")
    print(variety_no_futures_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/6/18 18:30
Desc: 新浪财经-债券-沪深债券-实时行情数据和历史行情数据
https://vip.stock.finance.sina.com.cn/mkt/#hs_z
"""

import datetime
import re

import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.bond.cons import (
    zh_sina_bond_hs_count_url,
    zh_sina_bond_hs_payload,
    zh_sina_bond_hs_url,
    zh_sina_bond_hs_hist_url,
)
from byteapp_ak.stock.cons import hk_js_decode
from byteapp_ak.utils import demjson
from byteapp_ak.utils.tqdm import get_tqdm


def get_zh_bond_hs_page_count() -> int:
    """
    行情中心首页-债券-沪深债券的总页数
    https://vip.stock.finance.sina.com.cn/mkt/#hs_z
    :return: 总页数
    :rtype: int
    """
    params = {
        "node": "hs_z",
    }
    res = requests.get(zh_sina_bond_hs_count_url, params=params)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def bond_zh_hs_spot(start_page: str = "1", end_page: str = "10") -> pd.DataFrame:
    """
    新浪财经-债券-沪深债券-实时行情数据, 大量抓取容易封IP
    https://vip.stock.finance.sina.com.cn/mkt/#hs_z
    :return: 所有沪深债券在当前时刻的实时行情数据
    :rtype: pandas.DataFrame
    """
    page_count = get_zh_bond_hs_page_count()
    page_count = int(page_count)
    zh_sina_bond_hs_payload_copy = zh_sina_bond_hs_payload.copy()
    tqdm = get_tqdm()
    big_df = pd.DataFrame()
    start_page = int(start_page)
    end_page = int(end_page) + 1 if int(end_page) + 1 <= page_count else page_count
    for page in tqdm(range(start_page, end_page), leave=False):
        zh_sina_bond_hs_payload_copy.update({"page": page})
        r = requests.get(zh_sina_bond_hs_url, params=zh_sina_bond_hs_payload_copy)
        data_json = demjson.decode(r.text)
        temp_df = pd.DataFrame(data_json)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "代码",
        "-",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "买入",
        "卖出",
        "昨收",
        "今开",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "买入",
            "卖出",
            "昨收",
            "今开",
            "最高",
            "最低",
            "成交量",
            "成交额",
        ]
    ]
    big_df["买入"] = pd.to_numeric(big_df["买入"], errors="coerce")
    big_df["卖出"] = pd.to_numeric(big_df["卖出"], errors="coerce")
    big_df["昨收"] = pd.to_numeric(big_df["昨收"], errors="coerce")
    big_df["今开"] = pd.to_numeric(big_df["今开"], errors="coerce")
    big_df["最高"] = pd.to_numeric(big_df["最高"], errors="coerce")
    big_df["最低"] = pd.to_numeric(big_df["最低"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    return big_df


def bond_zh_hs_daily(symbol: str = "sh010107") -> pd.DataFrame:
    """
    新浪财经-债券-沪深债券-历史行情数据, 大量抓取容易封 IP
    https://vip.stock.finance.sina.com.cn/mkt/#hs_z
    :param symbol: 沪深债券代码; e.g., sh010107
    :type symbol: str
    :return: 指定沪深债券代码的日 K 线数据
    :rtype: pandas.DataFrame
    """
    r = requests.get(
        zh_sina_bond_hs_hist_url.format(
            symbol, datetime.datetime.now().strftime("%Y_%m_%d")
        )
    )
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", r.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行 js 解密代码
    data_df = pd.DataFrame(dict_list)
    data_df["date"] = pd.to_datetime(data_df["date"], errors="coerce").dt.date
    data_df["open"] = pd.to_numeric(data_df["open"], errors="coerce")
    data_df["high"] = pd.to_numeric(data_df["high"], errors="coerce")
    data_df["low"] = pd.to_numeric(data_df["low"], errors="coerce")
    data_df["close"] = pd.to_numeric(data_df["close"], errors="coerce")
    return data_df


if __name__ == "__main__":
    bond_zh_hs_spot_df = bond_zh_hs_spot(start_page="1", end_page="5")
    print(bond_zh_hs_spot_df)

    bond_zh_hs_daily_df = bond_zh_hs_daily(symbol="sh010107")
    print(bond_zh_hs_daily_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/8/28 15:30
Desc: 巨潮资讯-数据中心-评级预测-投资评级
https://webapi.cninfo.com.cn/#/thematicStatistics?name=%E6%8A%95%E8%B5%84%E8%AF%84%E7%BA%A7
"""

import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.datasets import get_ths_js


def _get_file_content_cninfo(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_rank_forecast_cninfo(date: str = "20230817") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-评级预测-投资评级
    https://webapi.cninfo.com.cn/#/thematicStatistics?name=%E6%8A%95%E8%B5%84%E8%AF%84%E7%BA%A7
    :param date: 查询日期
    :type date: str
    :return: 投资评级
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1089"
    params = {"tdate": "-".join([date[:4], date[4:6], date[6:]])}
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "证券简称",
        "发布日期",
        "前一次投资评级",
        "评级变化",
        "目标价格-上限",
        "是否首次评级",
        "投资评级",
        "研究员名称",
        "研究机构简称",
        "目标价格-下限",
        "证券代码",
    ]
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "发布日期",
            "研究机构简称",
            "研究员名称",
            "投资评级",
            "是否首次评级",
            "评级变化",
            "前一次投资评级",
            "目标价格-下限",
            "目标价格-上限",
        ]
    ]
    temp_df["目标价格-上限"] = pd.to_numeric(temp_df["目标价格-上限"], errors="coerce")
    temp_df["目标价格-下限"] = pd.to_numeric(temp_df["目标价格-下限"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_rank_forecast_cninfo_df = stock_rank_forecast_cninfo(date="20230817")
    print(stock_rank_forecast_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/3/17 18:20
Desc: 新浪财经-美股指数行情
https://stock.finance.sina.com.cn/usstock/quotes/.IXIC.html
"""

import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.stock.cons import (
    zh_js_decode,
)


def index_us_stock_sina(symbol: str = ".INX") -> pd.DataFrame:
    """
    新浪财经-美股指数行情
    https://stock.finance.sina.com.cn/usstock/quotes/.IXIC.html
    :param symbol: choice of {".IXIC", ".DJI", ".INX", ".NDX"}
    :type symbol: str
    :return: 美股指数行情
    :rtype: pandas.DataFrame
    """
    url = f"https://finance.sina.com.cn/staticdata/us/{symbol}"
    r = requests.get(url)
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(zh_js_decode)
    dict_list = js_code.call("d", r.text.split("=")[1].split(";")[0].replace('"', ""))
    temp_df = pd.DataFrame(dict_list)
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["volume"] = pd.to_numeric(temp_df["volume"], errors="coerce")
    temp_df["amount"] = pd.to_numeric(temp_df["amount"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    index_us_stock_sina_df = index_us_stock_sina(symbol=".INX")
    print(index_us_stock_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/24 15:00
Desc: 申万宏源研究-申万指数-指数发布
乐咕乐股网
https://legulegu.com/stockdata/index-composition?industryCode=851921.SI
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup

from byteapp_ak.utils.cons import headers


def sw_index_first_info() -> pd.DataFrame:
    """
    乐咕乐股-申万一级-分类
    https://legulegu.com/stockdata/sw-industry-overview#level1
    :return: 分类
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/stockdata/sw-industry-overview"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    code_raw = soup.find(name="div", attrs={"id": "level1Items"}).find_all(
        name="div", attrs={"class": "lg-industries-item-chinese-title"}
    )
    name_raw = soup.find(name="div", attrs={"id": "level1Items"}).find_all(
        name="div", attrs={"class": "lg-industries-item-number"}
    )
    value_raw = soup.find(name="div", attrs={"id": "level1Items"}).find_all(
        name="div", attrs={"class": "lg-sw-industries-item-value"}
    )
    code = [item.get_text() for item in code_raw]
    name = [item.get_text().split("(")[0] for item in name_raw]
    num = [item.get_text().split("(")[1].split(")")[0] for item in name_raw]
    num_1 = [
        item.find_all("span", attrs={"class": "value"})[0].get_text().strip()
        for item in value_raw
    ]
    num_2 = [
        item.find_all("span", attrs={"class": "value"})[1].get_text().strip()
        for item in value_raw
    ]
    num_3 = [
        item.find_all("span", attrs={"class": "value"})[2].get_text().strip()
        for item in value_raw
    ]
    num_4 = [
        item.find_all("span", attrs={"class": "value"})[3].get_text().strip()
        for item in value_raw
    ]
    temp_df = pd.DataFrame([code, name, num, num_1, num_2, num_3, num_4]).T
    temp_df.columns = [
        "行业代码",
        "行业名称",
        "成份个数",
        "静态市盈率",
        "TTM(滚动)市盈率",
        "市净率",
        "静态股息率",
    ]
    temp_df["成份个数"] = pd.to_numeric(temp_df["成份个数"], errors="coerce")
    temp_df["静态市盈率"] = pd.to_numeric(temp_df["静态市盈率"], errors="coerce")
    temp_df["TTM(滚动)市盈率"] = pd.to_numeric(
        temp_df["TTM(滚动)市盈率"], errors="coerce"
    )
    temp_df["市净率"] = pd.to_numeric(temp_df["市净率"], errors="coerce")
    temp_df["静态股息率"] = pd.to_numeric(temp_df["静态股息率"], errors="coerce")
    return temp_df


def sw_index_second_info() -> pd.DataFrame:
    """
    乐咕乐股-申万二级-分类
    https://legulegu.com/stockdata/sw-industry-overview#level1
    :return: 分类
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/stockdata/sw-industry-overview"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    code_raw = soup.find(name="div", attrs={"id": "level2Items"}).find_all(
        name="div", attrs={"class": "lg-industries-item-chinese-title"}
    )
    name_raw = soup.find(name="div", attrs={"id": "level2Items"}).find_all(
        name="div", attrs={"class": "lg-industries-item-number"}
    )
    value_raw = soup.find(name="div", attrs={"id": "level2Items"}).find_all(
        name="div", attrs={"class": "lg-sw-industries-item-value"}
    )
    code = [item.get_text() for item in code_raw]
    name = [item.get_text().split("(")[0] for item in name_raw]
    num = [item.get_text().split("(")[1].split(")")[0] for item in name_raw]
    num_1 = [
        item.find_all("span", attrs={"class": "value"})[0].get_text().strip()
        for item in value_raw
    ]
    num_2 = [
        item.find_all("span", attrs={"class": "value"})[1].get_text().strip()
        for item in value_raw
    ]
    num_3 = [
        item.find_all("span", attrs={"class": "value"})[2].get_text().strip()
        for item in value_raw
    ]
    num_4 = [
        item.find_all("span", attrs={"class": "value"})[3].get_text().strip()
        for item in value_raw
    ]
    temp_df = pd.DataFrame([code, name, num, num_1, num_2, num_3, num_4]).T
    temp_df.columns = [
        "行业代码",
        "行业名称",
        "成份个数",
        "静态市盈率",
        "TTM(滚动)市盈率",
        "市净率",
        "静态股息率",
    ]
    temp_df["成份个数"] = pd.to_numeric(temp_df["成份个数"], errors="coerce")
    temp_df["静态市盈率"] = pd.to_numeric(temp_df["静态市盈率"], errors="coerce")
    temp_df["TTM(滚动)市盈率"] = pd.to_numeric(
        temp_df["TTM(滚动)市盈率"], errors="coerce"
    )
    temp_df["市净率"] = pd.to_numeric(temp_df["市净率"], errors="coerce")
    temp_df["静态股息率"] = pd.to_numeric(temp_df["静态股息率"], errors="coerce")
    return temp_df


def sw_index_third_info() -> pd.DataFrame:
    """
    乐咕乐股-申万三级-分类
    https://legulegu.com/stockdata/sw-industry-overview#level1
    :return: 分类
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/stockdata/sw-industry-overview"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    code_raw = soup.find(name="div", attrs={"id": "level3Items"}).find_all(
        name="div", attrs={"class": "lg-industries-item-chinese-title"}
    )
    name_raw = soup.find(name="div", attrs={"id": "level3Items"}).find_all(
        name="div", attrs={"class": "lg-industries-item-number"}
    )
    value_raw = soup.find(name="div", attrs={"id": "level3Items"}).find_all(
        name="div", attrs={"class": "lg-sw-industries-item-value"}
    )
    code = [item.get_text() for item in code_raw]
    name = [item.get_text().split("(")[0] for item in name_raw]
    num = [item.get_text().split("(")[1].split(")")[0] for item in name_raw]
    num_1 = [
        item.find_all("span", attrs={"class": "value"})[0].get_text().strip()
        for item in value_raw
    ]
    num_2 = [
        item.find_all("span", attrs={"class": "value"})[1].get_text().strip()
        for item in value_raw
    ]
    num_3 = [
        item.find_all("span", attrs={"class": "value"})[2].get_text().strip()
        for item in value_raw
    ]
    num_4 = [
        item.find_all("span", attrs={"class": "value"})[3].get_text().strip()
        for item in value_raw
    ]
    temp_df = pd.DataFrame([code, name, num, num_1, num_2, num_3, num_4]).T
    temp_df.columns = [
        "行业代码",
        "行业名称",
        "成份个数",
        "静态市盈率",
        "TTM(滚动)市盈率",
        "市净率",
        "静态股息率",
    ]
    temp_df["成份个数"] = pd.to_numeric(temp_df["成份个数"], errors="coerce")
    temp_df["静态市盈率"] = pd.to_numeric(temp_df["静态市盈率"], errors="coerce")
    temp_df["TTM(滚动)市盈率"] = pd.to_numeric(
        temp_df["TTM(滚动)市盈率"], errors="coerce"
    )
    temp_df["市净率"] = pd.to_numeric(temp_df["市净率"], errors="coerce")
    temp_df["静态股息率"] = pd.to_numeric(temp_df["静态股息率"], errors="coerce")
    return temp_df


def sw_index_third_cons(symbol: str = "801120.SI") -> pd.DataFrame:
    """
    乐咕乐股-申万三级-行业成份
    https://legulegu.com/stockdata/index-composition?industryCode=801120.SI
    :param symbol: 三级行业的行业代码
    :type symbol: str
    :return: 行业成份
    :rtype: pandas.DataFrame
    """
    url = f"https://legulegu.com/stockdata/index-composition?industryCode={symbol}"
    r = requests.get(url, headers=headers)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "纳入时间",
        "申万1级",
        "申万2级",
        "申万3级",
        "价格",
        "市盈率",
        "市盈率ttm",
        "市净率",
        "股息率",
        "市值",
        "归母净利润同比增长(09-30)",
        "归母净利润同比增长(06-30)",
        "营业收入同比增长(09-30)",
        "营业收入同比增长(06-30)",
    ]
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df["市盈率"] = pd.to_numeric(temp_df["市盈率"], errors="coerce")
    temp_df["市盈率ttm"] = pd.to_numeric(temp_df["市盈率ttm"], errors="coerce")
    temp_df["市净率"] = pd.to_numeric(temp_df["市净率"], errors="coerce")
    temp_df["股息率"] = pd.to_numeric(temp_df["股息率"].str.strip("%"), errors="coerce")
    temp_df["市值"] = pd.to_numeric(temp_df["市值"], errors="coerce")

    temp_df["归母净利润同比增长(09-30)"] = temp_df[
        "归母净利润同比增长(09-30)"
    ].str.strip("%")
    temp_df["归母净利润同比增长(06-30)"] = temp_df[
        "归母净利润同比增长(06-30)"
    ].str.strip("%")
    temp_df["营业收入同比增长(09-30)"] = temp_df["营业收入同比增长(09-30)"].str.strip(
        "%"
    )
    temp_df["营业收入同比增长(06-30)"] = temp_df["营业收入同比增长(06-30)"].str.strip(
        "%"
    )

    temp_df["归母净利润同比增长(09-30)"] = pd.to_numeric(
        temp_df["归母净利润同比增长(09-30)"], errors="coerce"
    )
    temp_df["归母净利润同比增长(06-30)"] = pd.to_numeric(
        temp_df["归母净利润同比增长(06-30)"], errors="coerce"
    )
    temp_df["营业收入同比增长(09-30)"] = pd.to_numeric(
        temp_df["营业收入同比增长(09-30)"], errors="coerce"
    )
    temp_df["营业收入同比增长(06-30)"] = pd.to_numeric(
        temp_df["营业收入同比增长(06-30)"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    sw_index_first_info_df = sw_index_first_info()
    print(sw_index_first_info_df)

    sw_index_second_info_df = sw_index_second_info()
    print(sw_index_second_info_df)

    sw_index_third_info_df = sw_index_third_info()
    print(sw_index_third_info_df)

    sw_index_third_cons_df = sw_index_third_cons(symbol="850111.SI")
    print(sw_index_third_cons_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/4/21 17:00
Desc: funddb-工具-估值情绪-恐惧贪婪指数
https://funddb.cn/tool/fear
"""

import json

import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.datasets import get_ths_js


def _get_file_content_ths(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf8") as f:
        file_data = f.read()
    return file_data


def index_fear_greed_funddb(symbol: str = "上证指数") -> pd.DataFrame:
    """
    funddb-工具-估值情绪-恐惧贪婪指数
    https://funddb.cn/tool/fear
    :param symbol: choice of {"上证指数", "沪深300"}
    :type symbol: str
    :return: 恐惧贪婪指数
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "上证指数": "000001.SH",
        "沪深300": "000300.SH",
    }
    url = "https://api.jiucaishuo.com/v2/kjtl/kjtlconnect"
    payload = {
        "gu_code": symbol_map[symbol],
        "type": "h5",
        "version": "2.4.5",
        "act_time": 1697623588394,
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("new_my_decode", data_json)
    data_json = json.loads(mcode)
    date_list = data_json["data"]["xAxis"]["categories"]
    tl_list = data_json["data"]["series"][0]["data"]
    sz_list = data_json["data"]["series"][1]["data"]
    temp_df = pd.DataFrame([date_list, tl_list, sz_list]).T
    temp_df.columns = ["date", "fear", "index"]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["fear"] = pd.to_numeric(temp_df["fear"], errors="coerce")
    temp_df["index"] = pd.to_numeric(temp_df["index"], errors="coerce")

    temp_df["fear"] = temp_df["fear"].round(2)
    temp_df["index"] = temp_df["index"].round(2)
    return temp_df


if __name__ == "__main__":
    index_fear_greed_funddb_df = index_fear_greed_funddb(symbol="上证指数")
    print(index_fear_greed_funddb_df)

    index_fear_greed_funddb_df = index_fear_greed_funddb(symbol="沪深300")
    print(index_fear_greed_funddb_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/8/18 17:30
Desc: 首页-行情中心-涨停板行情-涨停股池
https://quote.eastmoney.com/ztb/detail#type=ztgc

涨停板行情专题为您展示了 6 个股票池，分别为：
1. 涨停股池：包含当日当前涨停的所有A股股票(不含未中断连续一字涨停板的新股)；
2. 昨日涨停股池：包含上一交易日收盘时涨停的所有A股股票(不含未中断连续一字涨停板的新股)；
3. 强势股池：包含创下60日新高或近期多次涨停的A股股票；
4. 次新股池：包含上市一年以内且中断了连续一字涨停板的A股股票；
5. 炸板股池：包含当日触及过涨停板且当前未封板的A股股票；
6. 跌停股池：包含当日当前跌停的所有A股股票。
注：涨停板行情专题统计不包含ST股票及科创板股票。
"""

from datetime import datetime, timedelta

import pandas as pd
import requests


def stock_zt_pool_em(date: str = "20241008") -> pd.DataFrame:
    """
    东方财富网-行情中心-涨停板行情-涨停股池
    https://quote.eastmoney.com/ztb/detail#type=ztgc
    :param date: 交易日
    :type date: str
    :return: 涨停股池
    :rtype: pandas.DataFrame
    """
    url = "https://push2ex.eastmoney.com/getTopicZTPool"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wz.ztzt",
        "Pageindex": "0",
        "pagesize": "10000",
        "sort": "fbt:asc",
        "date": date,
        "_": "1621590489736",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if data_json["data"] is None:
        return pd.DataFrame()
    if len(data_json["data"]["pool"]) == 0:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["pool"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.columns = [
        "序号",
        "代码",
        "_",
        "名称",
        "最新价",
        "涨跌幅",
        "成交额",
        "流通市值",
        "总市值",
        "换手率",
        "连板数",
        "首次封板时间",
        "最后封板时间",
        "封板资金",
        "炸板次数",
        "所属行业",
        "涨停统计",
    ]
    temp_df["涨停统计"] = (
        temp_df["涨停统计"].apply(lambda x: dict(x)["days"]).astype(str)
        + "/"
        + temp_df["涨停统计"].apply(lambda x: dict(x)["ct"]).astype(str)
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "涨跌幅",
            "最新价",
            "成交额",
            "流通市值",
            "总市值",
            "换手率",
            "封板资金",
            "首次封板时间",
            "最后封板时间",
            "炸板次数",
            "涨停统计",
            "连板数",
            "所属行业",
        ]
    ]
    temp_df["首次封板时间"] = temp_df["首次封板时间"].astype(str).str.zfill(6)
    temp_df["最后封板时间"] = temp_df["最后封板时间"].astype(str).str.zfill(6)
    temp_df["最新价"] = temp_df["最新价"] / 1000
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["流通市值"] = pd.to_numeric(temp_df["流通市值"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["封板资金"] = pd.to_numeric(temp_df["封板资金"], errors="coerce")
    temp_df["炸板次数"] = pd.to_numeric(temp_df["炸板次数"], errors="coerce")
    temp_df["连板数"] = pd.to_numeric(temp_df["连板数"], errors="coerce")
    return temp_df


def stock_zt_pool_previous_em(date: str = "20240415") -> pd.DataFrame:
    """
    东方财富网-行情中心-涨停板行情-昨日涨停股池
    https://quote.eastmoney.com/ztb/detail#type=zrzt
    :param date: 交易日
    :type date: str
    :return: 昨日涨停股池
    :rtype: pandas.DataFrame
    """
    url = "https://push2ex.eastmoney.com/getYesterdayZTPool"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wz.ztzt",
        "Pageindex": "0",
        "pagesize": "170",
        "sort": "zs:desc",
        "date": date,
        "_": "1621590489736",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if data_json["data"] is None:
        return pd.DataFrame()
    if len(data_json["data"]["pool"]) == 0:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["pool"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.columns = [
        "序号",
        "代码",
        "_",
        "名称",
        "最新价",
        "涨停价",
        "涨跌幅",
        "成交额",
        "流通市值",
        "总市值",
        "换手率",
        "振幅",
        "涨速",
        "昨日封板时间",
        "昨日连板数",
        "所属行业",
        "涨停统计",
    ]
    temp_df["涨停统计"] = (
        temp_df["涨停统计"].apply(lambda x: dict(x)["days"]).astype(str)
        + "/"
        + temp_df["涨停统计"].apply(lambda x: dict(x)["ct"]).astype(str)
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "涨跌幅",
            "最新价",
            "涨停价",
            "成交额",
            "流通市值",
            "总市值",
            "换手率",
            "涨速",
            "振幅",
            "昨日封板时间",
            "昨日连板数",
            "涨停统计",
            "所属行业",
        ]
    ]
    temp_df["最新价"] = temp_df["最新价"] / 1000
    temp_df["涨停价"] = temp_df["涨停价"] / 1000
    temp_df["昨日封板时间"] = temp_df["昨日封板时间"].astype(str).str.zfill(6)
    return temp_df


def stock_zt_pool_strong_em(date: str = "20241009") -> pd.DataFrame:
    """
    东方财富网-行情中心-涨停板行情-强势股池
    https://quote.eastmoney.com/ztb/detail#type=qsgc
    :param date: 交易日
    :type date: str
    :return: 强势股池
    :rtype: pandas.DataFrame
    """
    url = "https://push2ex.eastmoney.com/getTopicQSPool"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wz.ztzt",
        "Pageindex": "0",
        "pagesize": "170",
        "sort": "zdp:desc",
        "date": date,
        "_": "1621590489736",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if data_json["data"] is None:
        return pd.DataFrame()
    if len(data_json["data"]["pool"]) == 0:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["pool"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.columns = [
        "序号",
        "代码",
        "_",
        "名称",
        "最新价",
        "涨停价",
        "_",
        "涨跌幅",
        "成交额",
        "流通市值",
        "总市值",
        "换手率",
        "是否新高",
        "入选理由",
        "量比",
        "涨速",
        "涨停统计",
        "所属行业",
    ]
    temp_df["涨停统计"] = (
        temp_df["涨停统计"].apply(lambda x: dict(x)["days"]).astype(str)
        + "/"
        + temp_df["涨停统计"].apply(lambda x: dict(x)["ct"]).astype(str)
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "涨跌幅",
            "最新价",
            "涨停价",
            "成交额",
            "流通市值",
            "总市值",
            "换手率",
            "涨速",
            "是否新高",
            "量比",
            "涨停统计",
            "入选理由",
            "所属行业",
        ]
    ]
    temp_df["最新价"] = temp_df["最新价"] / 1000
    temp_df["涨停价"] = temp_df["涨停价"] / 1000
    return temp_df


def stock_zt_pool_sub_new_em(date: str = "20241011") -> pd.DataFrame:
    """
    东方财富网-行情中心-涨停板行情-次新股池
    https://quote.eastmoney.com/ztb/detail#type=cxgc
    :param date: 交易日
    :type date: str
    :return: 次新股池
    :rtype: pandas.DataFrame
    """
    url = "https://push2ex.eastmoney.com/getTopicCXPooll"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wz.ztzt",
        "Pageindex": "0",
        "pagesize": "170",
        "sort": "ods:asc",
        "date": date,
        "_": "1621590489736",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if len(data_json["data"]["pool"]) == 0:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["pool"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.columns = [
        "序号",
        "代码",
        "_",
        "名称",
        "最新价",
        "涨停价",
        "_",
        "涨跌幅",
        "成交额",
        "流通市值",
        "总市值",
        "转手率",
        "开板几日",
        "开板日期",
        "上市日期",
        "_",
        "是否新高",
        "涨停统计",
        "所属行业",
    ]
    temp_df["涨停统计"] = (
        temp_df["涨停统计"].apply(lambda x: dict(x)["days"]).astype(str)
        + "/"
        + temp_df["涨停统计"].apply(lambda x: dict(x)["ct"]).astype(str)
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "涨跌幅",
            "最新价",
            "涨停价",
            "成交额",
            "流通市值",
            "总市值",
            "转手率",
            "开板几日",
            "开板日期",
            "上市日期",
            "是否新高",
            "涨停统计",
            "所属行业",
        ]
    ]
    temp_df["最新价"] = temp_df["最新价"] / 1000
    temp_df["涨停价"] = temp_df["涨停价"] / 1000
    temp_df.loc[temp_df["涨停价"] > 100000, "涨停价"] = pd.NA
    temp_df["开板日期"] = pd.to_datetime(temp_df["开板日期"], format="%Y%m%d")
    temp_df["上市日期"] = pd.to_datetime(temp_df["上市日期"], format="%Y%m%d")
    temp_df.loc[temp_df["上市日期"] == 0, "上市日期"] = pd.NaT
    return temp_df


def stock_zt_pool_zbgc_em(date: str = "20241011") -> pd.DataFrame:
    """
    东方财富网-行情中心-涨停板行情-炸板股池
    https://quote.eastmoney.com/ztb/detail#type=zbgc
    :param date: 交易日
    :type date: str
    :return: 炸板股池
    :rtype: pandas.DataFrame
    """
    thirty_days_ago = datetime.now() - timedelta(days=30)
    thirty_days_ago_str = thirty_days_ago.strftime("%Y%m%d")
    if int(date) < int(thirty_days_ago_str):
        raise ValueError("炸板股池只能获取最近 30 个交易日的数据")

    url = "https://push2ex.eastmoney.com/getTopicZBPool"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wz.ztzt",
        "Pageindex": "0",
        "pagesize": "170",
        "sort": "fbt:asc",
        "date": date,
        "_": "1621590489736",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if data_json["data"] is None:
        return pd.DataFrame()
    if len(data_json["data"]["pool"]) == 0:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["pool"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.columns = [
        "序号",
        "代码",
        "_",
        "名称",
        "最新价",
        "涨停价",
        "涨跌幅",
        "成交额",
        "流通市值",
        "总市值",
        "换手率",
        "首次封板时间",
        "炸板次数",
        "振幅",
        "涨速",
        "涨停统计",
        "所属行业",
    ]
    temp_df["涨停统计"] = (
        temp_df["涨停统计"].apply(lambda x: dict(x)["days"]).astype(str)
        + "/"
        + temp_df["涨停统计"].apply(lambda x: dict(x)["ct"]).astype(str)
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "涨跌幅",
            "最新价",
            "涨停价",
            "成交额",
            "流通市值",
            "总市值",
            "换手率",
            "涨速",
            "首次封板时间",
            "炸板次数",
            "涨停统计",
            "振幅",
            "所属行业",
        ]
    ]
    temp_df["最新价"] = temp_df["最新价"] / 1000
    temp_df["涨停价"] = temp_df["涨停价"] / 1000
    temp_df["首次封板时间"] = temp_df["首次封板时间"].astype(str).str.zfill(6)
    return temp_df


def stock_zt_pool_dtgc_em(date: str = "20241011") -> pd.DataFrame:
    """
    东方财富网-行情中心-涨停板行情-跌停股池
    https://quote.eastmoney.com/ztb/detail#type=dtgc
    :param date: 交易日
    :type date: str
    :return: 跌停股池
    :rtype: pandas.DataFrame
    """
    thirty_days_ago = datetime.now() - timedelta(days=30)
    thirty_days_ago_str = thirty_days_ago.strftime("%Y%m%d")
    if int(date) < int(thirty_days_ago_str):
        raise ValueError("跌停股池只能获取最近 30 个交易日的数据")

    url = "https://push2ex.eastmoney.com/getTopicDTPool"
    params = {
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "dpt": "wz.ztzt",
        "Pageindex": "0",
        "pagesize": "10000",
        "sort": "fund:asc",
        "date": date,
        "_": "1621590489736",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if len(data_json["data"]["pool"]) == 0:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["pool"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.columns = [
        "序号",
        "代码",
        "_",
        "名称",
        "最新价",
        "涨跌幅",
        "成交额",
        "流通市值",
        "总市值",
        "动态市盈率",
        "换手率",
        "封单资金",
        "最后封板时间",
        "板上成交额",
        "连续跌停",
        "开板次数",
        "所属行业",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "涨跌幅",
            "最新价",
            "成交额",
            "流通市值",
            "总市值",
            "动态市盈率",
            "换手率",
            "封单资金",
            "最后封板时间",
            "板上成交额",
            "连续跌停",
            "开板次数",
            "所属行业",
        ]
    ]
    temp_df["最新价"] = temp_df["最新价"] / 1000
    temp_df["最后封板时间"] = temp_df["最后封板时间"].astype(str).str.zfill(6)
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["流通市值"] = pd.to_numeric(temp_df["流通市值"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["动态市盈率"] = pd.to_numeric(temp_df["动态市盈率"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["封单资金"] = pd.to_numeric(temp_df["封单资金"], errors="coerce")
    temp_df["板上成交额"] = pd.to_numeric(temp_df["板上成交额"], errors="coerce")
    temp_df["连续跌停"] = pd.to_numeric(temp_df["连续跌停"], errors="coerce")
    temp_df["开板次数"] = pd.to_numeric(temp_df["开板次数"], errors="coerce")
    temp_df["开板次数"] = pd.to_numeric(temp_df["开板次数"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_zt_pool_em_df = stock_zt_pool_em(date="20241008")
    print(stock_zt_pool_em_df)

    stock_zt_pool_previous_em_df = stock_zt_pool_previous_em(date="20240415")
    print(stock_zt_pool_previous_em_df)

    stock_zt_pool_strong_em_df = stock_zt_pool_strong_em(date="20241009")
    print(stock_zt_pool_strong_em_df)

    stock_zt_pool_sub_new_em_df = stock_zt_pool_sub_new_em(date="20241011")
    print(stock_zt_pool_sub_new_em_df)

    stock_zt_pool_zbgc_em_df = stock_zt_pool_zbgc_em(date="20241011")
    print(stock_zt_pool_zbgc_em_df)

    stock_zt_pool_dtgc_em_df = stock_zt_pool_dtgc_em(date="20241011")
    print(stock_zt_pool_dtgc_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/6/18 18:30
Desc: 巨潮资讯-数据中心-行业分析-行业市盈率
http://webapi.cninfo.com.cn/#/thematicStatistics?name=%E6%8A%95%E8%B5%84%E8%AF%84%E7%BA%A7
"""

import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.datasets import get_ths_js


def _get_file_content_ths(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_industry_pe_ratio_cninfo(
    symbol: str = "证监会行业分类", date: str = "20210910"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-行业分析-行业市盈率
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param symbol: choice of {"证监会行业分类", "国证行业分类"}
    :type symbol: str
    :param date: 查询日期
    :type date: str
    :return: 行业市盈率
    :rtype: pandas.DataFrame
    """
    sort_code_map = {"证监会行业分类": "008001", "国证行业分类": "008200"}
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1087"
    params = {
        "tdate": "-".join([date[:4], date[4:6], date[6:]]),
        "sortcode": sort_code_map[symbol],
    }
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Accept-Enckey": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "行业层级",
        "静态市盈率-算术平均",
        "静态市盈率-中位数",
        "静态市盈率-加权平均",
        "净利润-静态",
        "行业名称",
        "行业编码",
        "行业分类",
        "总市值-静态",
        "纳入计算公司数量",
        "变动日期",
        "公司数量",
    ]
    temp_df = temp_df[
        [
            "变动日期",
            "行业分类",
            "行业层级",
            "行业编码",
            "行业名称",
            "公司数量",
            "纳入计算公司数量",
            "总市值-静态",
            "净利润-静态",
            "静态市盈率-加权平均",
            "静态市盈率-中位数",
            "静态市盈率-算术平均",
        ]
    ]
    temp_df["行业层级"] = pd.to_numeric(temp_df["行业层级"], errors="coerce")
    temp_df["公司数量"] = pd.to_numeric(temp_df["公司数量"], errors="coerce")
    temp_df["纳入计算公司数量"] = pd.to_numeric(
        temp_df["纳入计算公司数量"], errors="coerce"
    )
    temp_df["总市值-静态"] = pd.to_numeric(temp_df["总市值-静态"], errors="coerce")
    temp_df["净利润-静态"] = pd.to_numeric(temp_df["净利润-静态"], errors="coerce")
    temp_df["静态市盈率-加权平均"] = pd.to_numeric(
        temp_df["静态市盈率-加权平均"], errors="coerce"
    )
    temp_df["静态市盈率-中位数"] = pd.to_numeric(
        temp_df["静态市盈率-中位数"], errors="coerce"
    )
    temp_df["静态市盈率-算术平均"] = pd.to_numeric(
        temp_df["静态市盈率-算术平均"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    stock_industry_pe_ratio_cninfo_df = stock_industry_pe_ratio_cninfo(
        symbol="国证行业分类", date="20240617"
    )
    print(stock_industry_pe_ratio_cninfo_df)
